#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	<ctype.h>
#include	<bios.h>
#include	<time.h>
#include <Assert.h>

#include	"redir.h"

/****************************************************************************
 *                                                                          *
 *      INT 2F Function 11h File Redirector Support Library - 2fsubs.c      *
 *                                                                          *
 *     Copyright (c) 1992-1995 Printing Communications Associates, Inc.     *
 *                                                                          *
 *                            Thomas F. Divine                              *
 *                          4201 Brunswick Court                            *
 *                       Smyrna, Georgia 30080 USA                          *
 *                           TEL: 770 436-1714                              *
 *                           FAX: 770 436-7246                              *
 *                           CIS: 71077,1525                                *
 *                           AppleLink: PCA                                 *
 *                                                                          *
 ****************************************************************************/

/*
 * $Workfile:   2fsubs.c  $
 *  $Logfile:   D:/USR/SRC/NET/AFP/CPHANTOM/VCS/2FSUBS.C_V  $
 *   $Author:   PCA-TFD-SCSI  $
 *     $Date:   12 Jun 1992 12:55:22  $
 * $Revision:   1.4  $
 *
 * $Log:   D:/USR/SRC/NET/AFP/CPHANTOM/VCS/2FSUBS.C_V  $
 * 
 *    Rev 1.4   12 Jun 1992 12:55:22   PCA-TFD-SCSI
 * In fsCallForUs(), validity check for rdrFindNext was incorrect.
 * 
 *    Rev 1.3   15 May 1992 13:35:26   PCA-TFD-SCSI
 * 1.) Moved isCallForUs() from cphantom.c to this module. New name
 *     is fsCallForUs().
 * 
 *    Rev 1.2   13 May 1992 23:48:36   PCA-TFD-SCSI
 * 1.) In fsInitCDS(), corrected bug in "drive already in use" logical
 *     comparison.
 * 2.) Added fsSetSFTOwner() and fsDecSFTUseCount() routines.
 * 
 *    Rev 1.1   11 May 1992 23:46:22   PCA-TFD-SCSI
 * Added fsDate() and fsTime() routines.
 * 
 *    Rev 1.0   09 May 1992 11:15:16   PCA-TFD-SCSI
 * Initial revision.
 */

/* Global Variables
------------------- */
SDA3 far			*sda3Ptr;	// Pointer To DOS 3.XX Swappable Data Area(SDA)
SDA4 far			*sda4Ptr;	// Pointer To DOS 4.XX Swappable Data Area(SDA)

SFT31 far		*sftPtr;		// Current System File Table(SFT) Pointer

static SYSVARS	far *svPtr;		// Pointer To SYSVARS (List-Of-Lists)
static Uint16	CDSSize;		// Size Of CDS For Current DOS Version

union REGS		regs;			// General Use
struct SREGS	sregs;		// General Use

REGSET			*rsPtr;		// Setup On Entry To Register Set


/* Components And Template For Path Number 1
-------------------------------------------- */
Uint8		FN1Prefix[ TP_MAXPREFIX ];
Uint8		FN1Drive[ MAXDRIVE ];
Uint8		FN1Dir[ MAXDIR ];
Uint8		FN1FileName[ MAXFILE + MAXEXT ];	// File Name, '.', Plus Extension

Uint8		FN1Template[ 12 ];		// Template Buffer No. 1


/* Components And Template For Path Number 2
-------------------------------------------- */
Uint8		FN2Prefix[ TP_MAXPREFIX ];
Uint8		FN2Drive[ MAXDRIVE ];
Uint8		FN2Dir[ MAXDIR ];
Uint8		FN2FileName[ MAXFILE + MAXEXT ];	// File Name, '.', Plus Extension

Uint8		FN2Template[ 12 ];		// Template Buffer No. 2


/* Get DOS Variables
--------------------
 * Returns 0 on success, negative error code on failure.
 */
int fsGetDOSVars( void )
{
	/* Check DOS Version
	-------------------- */
	if( (_osmajor < 3) || ( (_osmajor == 3) && (_osminor < 10) ) )
	{
		return( -1 );
	}

	/* Perform INT 2F Installation Check
	------------------------------------ */
	regs.x.ax = 0x1100;

	int86( 0x2F, &regs, &regs );

	if( regs.h.al == 0x01 )
	{
		return( -2 );
	}

	/* Get Pointer To Swappable Data Area(SDA)
	------------------------------------------ */
	regs.x.ax = 0x5D06;

	intdosx( &regs, &regs, &sregs );

	sda3Ptr =(SDA3 far * ) MK_FP( sregs.ds, regs.x.si );
	sda4Ptr =(SDA4 far * ) MK_FP( sregs.ds, regs.x.si );


	/* Get Pointer To SYSVARS (List-Of-Lists)
	----------------------------------------- */
	regs.h.ah = 0x52;
	sregs.es = regs.x.bx = 0;

	intdosx( &regs, &regs, &sregs );

	if( !sregs.es && regs.x.bx )
	{
		return( -3 );
	}

	svPtr = (SYSVARS far * )MK_FP( sregs.es, regs.x.bx - SYSVARS_DECR );


	/* Compute CDS Size For Current DOS Version
	------------------------------------------- */
	CDSSize = (_osmajor >= 4) ? 0x58 : 0x51;

	return( 0 );
}


/* Initialize The Drive's Current Directory Structure(CDS)
---------------------------------------------------------- */
int fsInitCDS( Uint8 driveNum )
{
	Uint8 far	*uPtr;
	CDS4 far		*driveCDSPtr;	// Pointer To Drive's CDS

	/* Validate Drive Against LASTDRIVE Value
	----------------------------------------- */
	if( driveNum > svPtr->vers.dos31.lastdrive )
	{
		return( -1 );
	}


	/* Find Drive's Current Directory Structure(CDS)
	------------------------------------------------ */
	uPtr = (Uint8 far * )svPtr->vers.dos31.currdir;
	uPtr += ( CDSSize * ( driveNum - 1 ) );

	driveCDSPtr = (CDS4 far * )uPtr;


	/* Don't Install If Drive Is Already In Use
	------------------------------------------- */
	if( driveCDSPtr->flags & ( NETWORK | PHYSICAL ) )
	{
		return( -2 );
	}


	/* Edit The Drive's CDS
	----------------------- */
	driveCDSPtr->flags |= ( NETWORK | PHYSICAL );

	sprintf( (char far * )driveCDSPtr->curr_path, "%s%c:\\",
		SIGNATURE, driveNum + '@' );


	/* Calculate Offset To '\' Of Root Path
	--------------------------------------- */
	driveCDSPtr->root_offset = strlen( (char far * )driveCDSPtr->curr_path ) - 1;

	return( 0 );
}


/* Setup Registers To Indicate Call Failure And Return Error Code
----------------------------------------------------------------- */
void fsFailCall( Uint16 ErrCode )
{
	rsPtr->x.flags |= fcarry;
	rsPtr->x.ax = ErrCode;
}


/* Split A TrueName Path Into Components
----------------------------------------
 * This is similar to the _splitpath library routine. The key differences
 * are 1.) SplitPath returns the TrueName prefix (i.e., characters
 * preceeding the '\' representing the root directory) in the PREFIX buffer,
 * and 2.) the FNAME buffer will include the filename and extension.
 */
void fsSplitPath( Uint8 far *path,
			Uint8 *prefix, Uint8 *drive, Uint8 *dir, Uint8 *fname )
{
	Uint8	far *s, Extent[ MAXEXT ];

	/* Extract Prefix String
	------------------------ */
	if( prefix )
	{
		memcpy( prefix, path, strlen( SIGNATURE ) );
		prefix[ strlen( SIGNATURE ) ] = '\0';
	}


	/* Point To First Character Beyond SIGNATURE
	-------------------------------------------- */
	s = path;
	s += strlen( SIGNATURE );


	/* Let The Library Routines Do This Work
	---------------------------------------- */
	_splitpath( (char far * )s, (char * )drive, (char * )dir, (char * )fname, (char * )Extent );


	/* Concatenate _splitpath FileName And Extension
	------------------------------------------------ */
	if( fname )
	{
		strcat( (char * )fname, (char * )Extent );
	}
}


/* Verify TruePath Prefix And Return Our Drive Number
-----------------------------------------------------
 * A prefix can preceed the '\' delimiter which represents the root of
 * the file system. Information in the prefix is part of fully-qualified
 * paths (i.e., "truenames"). However, the prefix is not normally shown
 * to or used by the DOS user.
 *
 * The prefix can carry any information which may be useful to the redirector.
 * For example, in network systems it may contain the name of the remote
 * host:
 *
 * 	\\Billy\TestDir\Lemon\????????.???
 *           ^
 * In the CPhantom implementation, the prefix contains a signature and the
 * drive letter:
 * 	CPhantom-G:\TestDir\Lemon\????????.???
 *               ^
 * This routine simply verifies the CPhantom signature. It then converts
 * drive letter (A-Z) to the associated drive number (1-27).
 *
 * If the path contains our prefix, then the positive drive number is
 * returned. A zero result indicates that the path is not for us.
 *
 */
int fsOurDrive( Uint8 far *TruePath )
{
	int	i;
	Uint8		Prefix[ TP_MAXPREFIX ];
	Uint8		Drive[ MAXDRIVE ];
	Uint8		FileName[ MAXFILE + MAXEXT ];

	/* Split The Path
	----------------- */
	fsSplitPath( TruePath, Prefix, Drive, NULL, FileName );

	if( strcmp( (char * )FileName, "LPT3" ) == 0 )
	{
		i = 1;
	}

	if( strcmp( (char * )FileName, "LPT3:" ) == 0 )
	{
		i = 1;
	}

	/* Verify Our Signature Preceeding Root Path In TruePath
	-------------------------------------------------------- */
	if( strcmp( (char * )Prefix, SIGNATURE ) == 0 )
	{
		/* Return The Drive Number (A=1, etc)
		------------------------------------- */
		return( *Drive - '@' );
	}

	return( 0 );
}


/* Build An FCB-Style Search Template From Specified FileName
-------------------------------------------------------------
 * A FILENAME is an 8-character NAME, optionally followed by a '.'
 * delimiter and a 3-character EXTENSION. The FILENAME is terminated
 * by a '\0' null delimiter.
 *
 * A TEMPLATE is an 11-character array with the first 8 characters
 * representing the NAME and the last 3 characters representing the
 * EXTENSION.
 *
 * This routine builds a TEMPLATE based on the specified FILENAME. The
 * routine expands '*' and '?' wildcards. All characters of the TEMPLATE
 * are translated to upper case.
 */
void fsMakeTemplate( Uint8 *Template, Uint8 *FileName )
{
	int i;

	/* Handle The null FileName
	--------------------------- */
	if( !FileName || *FileName == 0 )
	{
		memset( Template, 0x00, 11 );
		return;
	}

	i = 0;


	/* Fill The Template NAME Characters
	------------------------------------ */
	while( i < 8 && *FileName != '\0' && *FileName != '.' )
	{
		if( *FileName == '*' )
		{
			while( i < 8 )
			{
				Template[ i++ ] = '?';
			}
		}
		else
		{
			if( i < 8 )
			{
				Template[ i++ ] = toupper( *FileName );
			}
		}
		++FileName;
	}


	/* Pad The Balance Of The Template NAME
	--------------------------------------- */
	while( i < 8 )
	{
		Template[ i++ ] = ' ';
	}


	/* Skip The Extension Delimiter
	------------------------------- */
	if( *FileName == '.' )
	{
		++FileName;
	}


	/* Fill The Template EXTENSION Characters
	----------------------------------------- */
	while( *FileName != '\0' )
	{
		if( *FileName == '*' )
		{
			while( i < 11 )
			{
				Template[ i++ ] = '?';
			}
		}
		else
		{
			if( i < 11 )
			{
				Template[ i++ ] = toupper( *FileName );
			}
		}

		++FileName;
	}

	/* Pad The Balance Of The Template EXTENSION
	-------------------------------------------- */
	while( i < 11 )
	{
		Template[ i++ ] = ' ';
	}
}


/* Match The FCB-Style Template Against The Specified FileName
-------------------------------------------------------------- */
int fsMatchTemplate( Uint8 *Template, Uint8 *FileName )
{
	static Uint8	temp2[11];
	int				i;

	if( !FileName || *FileName == '\0' )
	{
		return( 0 );
	}

	fsMakeTemplate( temp2, FileName );

	for( i = 0; i < 11; ++i )
	{
		if( Template[i] != temp2[i] && Template[i] != '?' )
		{
			return( 0 );
		}
	}

	return( 1 );
}


/* Get Current Date In FindXXXX Format From SDA
-----------------------------------------------
 * See DOS Technical Reference Manual for description of FindFirst
 * date format.
 */
Uint16 fsDate( void )
{
	Uint16	ans;

	ans = 0;

	ans = SDA_YY_1980 << 9;
	ans |= SDA_MM << 5;
	ans |= SDA_DD;

	return( ans );
}


/* Get Current Time Date In FindXXXX Format From BIOS Time-Of-Day
-----------------------------------------------------------------
 * See DOS Technical Reference Manual for description of FindFirst
 * time format.
 *
 * As each day progresses, the difference between using 18.2 ticks/sec
 * and 18 ticks/sec becomes noticable. Near midnight, the difference is
 * almost 24 minutes.
 */
Uint16 fsTime( void )
{
	Uint16	ans;
	long		bios_time, ltemp;

	_bios_timeofday( _TIME_GETCLOCK, &bios_time );

	/* Convert Ticks-To-Seconds - Without Floating Point
	---------------------------------------------------- */
	bios_time *= 10L;
	bios_time /= 182L;							// Same As bios_time/18.2

	ltemp = (bios_time/3600L) & 0x1F;		// Hours
	ans =  ltemp << 11L;

	bios_time -= ltemp * 3600L;

	ltemp = (bios_time/60L) & 0x3F;			// Minutes
	ans |= ltemp << 5L;

	bios_time -= ltemp * 60L;

	ltemp = (bios_time/2L) & 0x1F;			// 2-second Units
	ans |= ltemp;

	return( ans );
}


void fsSetSFTOwner( SFT31 far *theSFTPtr )
{
	regs.x.ax = 0x120C;

	segread( &sregs );

	sregs.es = FP_SEG( (void * )theSFTPtr );
	regs.x.di = FP_OFF( (void * )theSFTPtr );

	int86x( 0x2F, &regs, &regs, &sregs );
}


Uint16 fsDecSFTUseCount( SFT31 far *theSFTPtr )
{
	regs.x.ax = 0x1208;

	segread( &sregs );

	sregs.es = FP_SEG( (void * )theSFTPtr );
	regs.x.di = FP_OFF( (void * )theSFTPtr );

	int86x( 0x2F, &regs, &regs, &sregs );

	return( regs.x.ax );
}


/* Verify That This Call Is For Us
---------------------------------- */
int fsCallForUs( char DriveNo )
{
	int	isForUs;

	isForUs = 0;								// Assume Not For Us

	/* Switch On Subfunction In AL
	------------------------------ */
	switch( rsPtr->h.al )
	{
		case rdrInstCheck:
			isForUs = 1;						// This Call Is For Us
			break;

		/* Calls Dealing With An Open File
		---------------------------------- */
		case rdrCloseFile:
		case rdrCommitFile:
		case rdrRead:
		case rdrWrite:
		case rdrLockRegion:
		case rdrUnlockRegion:
		case rdrSeekFromEnd:
			/* Verify That SFP Is For Our Drive
			----------------------------------- */
			if( ( sftPtr->dev_info & 0x1F ) == DriveNo )
			{
				isForUs = 1;
			}
			break;

		/* Other Supported Calls
		------------------------ */
		case rdrRmDir:
		case rdrMkDir:
		case rdrChDir:
		case rdrGetDiskSpace:
		case rdrSetFileAttrib:
		case rdrGetFileAttrib:
		case rdrRenameFile:
		case rdrDeleteFile:
		case rdrOpenFile:
		case rdrCreateFile:
		case rdrFindFirst:
		case ifsSpecialOpen:
			/* SDA Contains Pointer To Drive CDS Structure
			---------------------------------------------- */
			if( fsOurDrive( SDA_CDSPTR->curr_path ) == DriveNo )
			{
				isForUs = 1;
			}
			break;

		case rdrFindNext:
			/* SDB Contains Drive Letter
			---------------------------- */
			if( ( SDA_SDB.drv_lett & 0x1F ) == DriveNo )
			{
				isForUs = 1;
			}
			break;

		default:
			break;					// Pretend Unsupported Calls Are Not For Us
	}

	return( isForUs );
}

