/****************************************************************************
 *                                                                          *
 *            Header For AFP Network INT 2F Redirector - redir.h            *
 *                                                                          *
 *     Copyright (c) 1992-1995 Printing Communications Associates, Inc.     *
 *                                                                          *
 *                            Thomas F. Divine                              *
 *                          4201 Brunswick Court                            *
 *                       Smyrna, Georgia 30080 USA                          *
 *                           TEL: 404 436-1714                              *
 *                           FAX: 404 436-7246                              *
 *                                                                          *
 ****************************************************************************/

/*
 * $Workfile:   redir.h  $
 *  $Logfile:   D:/USR/SRC/NET/AFP/CPHANTOM/VCS/REDIR.H_V  $
 *   $Author:   PCA-TFD-SCSI  $
 *     $Date:   11 Jun 1992 11:54:36  $
 * $Revision:   1.7  $
 *
 * $Log:   D:/USR/SRC/NET/AFP/CPHANTOM/VCS/REDIR.H_V  $
 * 
 *    Rev 1.7   11 Jun 1992 11:54:36   PCA-TFD-SCSI
 * Changed curr_dta type from dir_rec * to Uint8 *.
 * 
 *    Rev 1.6   15 May 1992 13:33:22   PCA-TFD-SCSI
 * 1.) Modifications to support compilation in COMPACT memory model.
 * 
 *    Rev 1.5   13 May 1992 23:42:36   PCA-TFD-SCSI
 * 1.) Changed signature to begin with "\\\\".
 * 2.) Removed unreferenced SFT20, SFT30, and SFT structure definitions.
 * 3.) Added SDA_SPOP_ACT, SDA_SPOP_ATTR, and SDA_SPOP_MODE macro definitions.
 * 
 *    Rev 1.4   11 May 1992 23:48:48   PCA-TFD-SCSI
 * 1.) In SDA's, exchanged dd and mm field order.
 * 2.) Added SDA_DD, SDA_MM, and SDA_YY_1980 macros.
 * 3.) Added fsDate() and fsTime() prototypes.
 * 
 *    Rev 1.3   11 May 1992 10:39:24   PCA-TFD-SCSI
 * 1.) Changed REGSET definition.
 * 2.) Added DOS error code definitions.
 * 
 *    Rev 1.2   09 May 1992 11:15:50   PCA-TFD-SCSI
 * 1.) Extensive revision. Added function prototypes.
 * 
 *    Rev 1.1   06 May 1992 13:20:06   PCA-TFD-SCSI
 * 1.) Added structure definitions for DOS 3.XX.
 * 2.) Added access macros to support operation under multiple DOS versions.
 * 
 *    Rev 1.0   05 May 1992 18:25:24   PCA-TFD-SCSI
 * Initial revision.
 */

#ifndef	__REDIR_H__
#define	__REDIR_H__

#if !defined( __DIR_H )
#include	<dir.h>
#endif

#ifndef __DOS_H
#include	<dos.h>
#endif

#ifndef	__PCATYPES_H
#include	"pcatypes.h"
//typedef unsigned char	    BYTE;
//typedef unsigned short      WORD;
//typedef unsigned long       DWORD;
#endif

#define SIGNATURE	"\\\\PCAAFP"

/* Register Set Definition
-------------------------- */
struct rhregs
{
	Uint16	bp, di, si, ds, es;
	Uint8		dl, dh, cl, ch, bl, bh, al, ah;
	Uint16	flags;
};

struct rxregs
{
	Uint16	bp, di, si, ds, es, dx, cx, bx, ax, flags;
};

typedef
union
{
	struct rhregs h;
	struct rxregs x;
}
	REGSET;

/* flags Definitions
-------------------- */
#define fcarry			1					// Carry Flag


/* DOS Error Codes (Borland C Names)
------------------------------------ */
#define EZERO    0      /* Error 0                  */
#define ENOPATH  3      /* Path not found           */
#define EACCES   5      /* Permission denied        */
#define ENMFILE 18      /* No more files            */


/* INT 2F Sub-Function(AL) Codes
-------------------------------- */
enum
{
//	Code							  DEC  HEX   Comment
	rdrInstCheck,				//  0, 00H - Installation Check
	rdrRmDir,					//  1, 01H - Remove Remote Directory
	ifsRmDir,					//  2, 02H - Remove Remote Directory (IFS)
	rdrMkDir,					//  3, 03H - Make Remote Directory
	ifsMkDir,					//  4, 04H - Make Remote Directory (IFS)
	rdrChDir,					//  5, 05H - Change Directory
	rdrCloseFile,				//  6, 06H - Close File
	rdrCommitFile,				//  7, 07H - Flush File
	rdrRead,						//  8, 08H - Read
	rdrWrite,					//  9, 09H - Write
	rdrLockRegion,				// 10, 0AH - Lock Region Of File
	rdrUnlockRegion,			// 11, 0BH - Unlock Region Of File
	rdrGetDiskSpace,			// 12, 0CH - Get Disk Space
	ifsUnknown1,				// 13, 0DH - Unknown (IFS)
	rdrSetFileAttrib,			// 14, 0EH - Set File Attributes
	rdrGetFileAttrib,			// 15, 0FH - Get File Attributes
	ifsUnknown2,				// 16, 10H - Unknown (IFS)
	rdrRenameFile,				// 17, 11H - Rename File
	ifsUnknown3,				// 18, 12H - Unknown (IFS)
	rdrDeleteFile,				// 19, 13H - Delete File
	ifsUnknown4,				// 20, 14H - Unknown (IFS)
	ifsUnknown5,				// 21, 15H - Unknown (IFS)
	rdrOpenFile,				// 22, 16H - Open Existing File
	rdrCreateFile,				// 23, 17H - Create/Truncate File
	rdrCreateFile2,			// 24, 18H - Create/Truncate File
	rdrUnknown1,				// 25, 19H - Unknown
	ifsUnknown6,				// 26, 1AH - Unknown (IFS)
	rdrFindFirst,				// 27, 1BH - Find First
	rdrFindNext,				// 28, 1CH - Find Next
	rdrCloseAll,				// 29, 1DH - Close All Remote Files For Process
	rdrDoRedirection,			// 30, 1EH - Do Redirection
	rdrPrinterSetup,			// 31, 1FH - Printer Setup
	rdrFlushBuffers,			// 32, 20H - Flush All Disk Buffers
	rdrSeekFromEnd,			// 33, 21H - Seek From End Of File
	rdrProcessDied,			// 34, 22H - Process Termination Hook
	rdrQualifyName,			// 35, 23H - Qualify File Name
	rdrUnknown2,				// 36, 24H - Unknown
	rdrPrinterMode,			// 37, 25H - Redirected Printer Mode
	rdrUnknown3,				// 38, 26H - Unknown
	ifsUnused1,					// 39, 27H - Unused (IFS)
	ifsUnused2,					// 40, 28H - Unused (IFS)
	ifsUnused3,					// 41, 29H - Unused (IFS)
	ifsUnknown7,				// 42, 2AH - Unknown (IFS)
	ifsUnknown8,				// 43, 2BH - Unknown (IFS)
	ifsUnknown9,				// 44, 2CH - Unknown (IFS)
	ifsUnknown10,				// 45, 2DH - Unknown (IFS)
	ifsSpecialOpen,			// 46, 2EH - Special Multi-Purpose Open File
	ifsUnknown12,				// 47, 2FH - Unknown (IFS)
	ifsGetIFSSegment			// 48, 30H - Get IFSFUNC Segment (IFS)
};

typedef Uint8 far			*DPB;		// provide actual DPB struct if needed

#pragma pack(1)

/* DOS V3.XX Current Directory Structure(CDS)
--------------------------------------------- */
typedef struct CDS3
{
	Uint8		curr_path[67];			// current path
	Uint16	flags;					// NETWORK, PHYSICAL, JOIN, SUBST
	DPB		dpb;						// pointer to Drive Parameter Block
	union
	{
		struct
		{
			Uint16	start_cluster;	// root: 0000; never accessed: FFFFh
			Uint32	unknown;
		} LOCAL;							// if (! (cds[drive].flags & NETWORK))
		struct
		{
			Uint32	redirifs_record_ptr;
			Uint16	parameter;
		} NET;							// if (cds[drive].flags & NETWORK)
	} u;
	Uint16			root_offset;	// offset in curr_path of '\'
}
	CDS3;

	
#pragma pack(1)

/* DOS V4.XX Current Directory Structure(CDS)
--------------------------------------------- */
typedef struct
{
	Uint8		curr_path[67];			// current path
	Uint16	flags;					// NETWORK, PHYSICAL, JOIN, SUBST
	DPB		dpb;						// pointer to Drive Parameter Block
	union
	{
		struct
		{
			Uint16	start_cluster;	// root: 0000; never accessed: FFFFh
			Uint32	unknown;
		} LOCAL;							// if (! (cds[drive].flags & NETWORK))
		struct
		{
			Uint32	redirifs_record_ptr;
			Uint16	parameter;
		} NET;							// if (cds[drive].flags & NETWORK)
	} u;
	Uint16			root_offset;	// offset in curr_path of '\'
	// DOS4 fields for IFS
	// 7 extra bytes...
	Uint8				fill2[7];
}
	CDS4;


/* Definitions For CDS flags Field
---------------------------------- */
#define NETWORK     (1 << 15)
#define PHYSICAL    (1 << 14)
#define JOIN        (1 << 13)
#define SUBST       (1 << 12)

	
#pragma pack(1)

#define SYSVARS_DECR        12

/* DOS SYSVARS Structure (Undocumented DOS "List-Of-Lists")
----------------------------------------------------------- */
typedef struct
{
	Uint16		shareretrycount;
	Uint16		shareretrydelay;
	void far		*currdiskbuff;
	Uint16		unreadcon;
	Uint16		mcb;
	void far		*dpb;
	void far		*filetable;
	void far		*clock;
	void far		*con;
	union
	{
		struct
		{
			Uint8		numdrive;
			Uint16	maxbytes;
			void far	*first_diskbuff;
			Uint8		nul[18];
		} dos2;
		struct
		{
			Uint8		numblkdev;
			Uint16	maxbytes;
			void far	*first_diskbuff;
			void far	*currdir;
			Uint8		lastdrive;
			void far *stringarea;
			Uint16	size_stringarea;
			void far	*fcbtab;
			Uint16	fcb_y;
			Uint8		nul[18];
		} dos30;
		struct
		{
			Uint16	maxbytes;
			void far *diskbuff;
			void far	*currdir;
			void far	*fcb;
			Uint16	numprotfcb;
			Uint8		numblkdev;
			Uint8		lastdrive;
			Uint8		nul[18];
			Uint16	numjoin;
		} dos31;    /* and higher */
	} vers;
}
	SYSVARS;

/* DOS System File Table
------------------------ */
#pragma pack(1)

typedef struct SFT31
{
	Uint16	handle_cnt, open_mode;
	Uint8		attr_byte;
	Uint16	dev_info;    // includes drive number
	Uint8 far	*devdrv_ptr;
	Uint16	start_clstr, f_time, f_date;
	Uint32	f_size, f_pos;
	Uint16	rel_lastclstr, abs_lastclstr, dir_sector;
	Uint8		dir_entryno;
	Uint8		filename[8], ext[3];
	Uint32	share_prev_sft;
	Uint16	share_net_machine;
	Uint16	owner_psp;
	// ...
}
	SFT31; // for DOS 3.1+, 4.x, see UNDOCUMENTED DOS, pp. 527-528

     
/* FindFirst Data Block - SDB
----------------------------- */
typedef struct sdb_rec
{
	Uint8		drv_lett;					// Drive Letter
	Uint8		srch_tmpl[11];				// Search Template
	Uint8		srch_attr;					// Search Attribute
	Uint16	dir_entry;					// Entry Count Within Directory
	Uint16	par_clstr;					// Cluster Number Of Parent Directory
	Uint8		f1[4];
}
	sdb_rec;

typedef struct dir_rec
{
	Uint8		fname[11];
	Uint8		fattr;
	Uint8		f1[10];
	Uint16	time_lstupd;
	Uint16	date_lstupd;
	Uint16	start_clstr;
	Uint32	fsiz;
}
	dir_rec;


/* DOS V3.XX Swappable DOS Area(SDA)
------------------------------------ */
typedef struct SDA3
{
	Uint8		f0[12];
	Uint8 far	*curr_dta;
	Uint8		f1[30];
	Uint8		dd;
	Uint8		mm;
	Uint16	yy_1980;
	Uint8		f2[96];
	Uint8		fn1[128];
	Uint8		fn2[128];
	sdb_rec	sdb;
	dir_rec	found_file;
	CDS3		drive_cdscopy;
	Uint8		fcb_fn1[11];
	Uint8		f3;
	Uint8		fcb_fn2[11];
	Uint8		f4[11];
	Uint8		srch_attr;
	Uint8		open_mode;
	Uint8		f5[48];
	CDS3 far	*drive_cdsptr;
	Uint8		f6[12];
	Uint16	fn1_csofs;
	Uint16	fn2_csofs;
	Uint8		f7[56];
	sdb_rec	ren_srcfile;
	dir_rec	ren_file;
}
	SDA3;


/* DOS V4.XX Swappable DOS Area(SDA)
------------------------------------ */
typedef struct SDA4
{
	Uint8		f0[12];
	Uint8 far	*curr_dta;
	Uint8		f1[32];
	Uint8		dd;
	Uint8		mm;
	Uint16	yy_1980;
	Uint8		f2[106];
	Uint8		fn1[128];
	Uint8		fn2[128];
	sdb_rec	sdb;
	dir_rec	found_file;
	CDS4		drive_cdscopy;
	Uint8		fcb_fn1[11];
	Uint8		f3;
	Uint8		fcb_fn2[11];
	Uint8		f4[11];
	Uint8		srch_attr;
	Uint8		open_mode;
	Uint8		f5[51];
	CDS4 far	*drive_cdsptr;
	Uint8		f6[12];
	Uint16	fn1_csofs;
	Uint16	fn2_csofs;
	Uint8		f7[71];
	Uint16	spop_act;
	Uint16	spop_attr;
	Uint16	spop_mode;
	Uint8		f8[29];
	sdb_rec	ren_srcfile;
	dir_rec	ren_file;
}
	SDA4;

/* Access Macros (Yeech!)
------------------------- */
#define SDA_FN1			( (_osmajor > 3) ? sda4Ptr->fn1 : sda3Ptr->fn1 )
#define SDA_FN2			( (_osmajor > 3) ? sda4Ptr->fn2 : sda3Ptr->fn2 )
#define SDA_SRCH_ATTR	( (_osmajor > 3) ? sda4Ptr->srch_attr : sda3Ptr->srch_attr )
#define SDA_OPEN_MODE	( (_osmajor > 3) ? sda4Ptr->open_mode : sda3Ptr->open_mode )

#define SDA_SPOP_ACT		( sda4Ptr->spop_act )
#define SDA_SPOP_ATTR	( sda4Ptr->spop_attr )
#define SDA_SPOP_MODE	( sda4Ptr->spop_mode )

#define SDA_FOUND_FILE	( (_osmajor > 3) ? sda4Ptr->found_file : sda3Ptr->found_file )
#define SDA_SDB			( (_osmajor > 3) ? sda4Ptr->sdb : sda3Ptr->sdb )
#define SDA_CDSPTR		( (_osmajor > 3) ? sda4Ptr->drive_cdsptr : (CDS4 * )sda3Ptr->drive_cdsptr )
#define SDA_DTA			( (_osmajor > 3) ? sda4Ptr->curr_dta : sda3Ptr->curr_dta )
#define SDA_DD				( (_osmajor > 3) ? sda4Ptr->dd : sda3Ptr->dd )
#define SDA_MM				( (_osmajor > 3) ? sda4Ptr->mm : sda3Ptr->mm )
#define SDA_YY_1980		( (_osmajor > 3) ? sda4Ptr->yy_1980 : sda3Ptr->yy_1980 )


/* Global Variables
------------------- */
extern SDA3 far	*sda3Ptr;	// Pointer To DOS 3.XX Swappable Data Area(SDA)
extern SDA4 far	*sda4Ptr;	// Pointer To DOS 4.XX Swappable Data Area(SDA)

extern SFT31 far	*sftPtr;		// Current System File Table(SFT) Pointer

extern union	REGS regs;		// General Use
extern struct	SREGS sregs;	// General Use

extern REGSET	*rsPtr;			// Setup On Entry To Register Set


/* Definitions For SplitPath Prefix Length
------------------------------------------ */
#define	TP_MAXPREFIX	10				// Depends On Choice Of Prefix


/* Components And Template For Path Number 1
-------------------------------------------- */
extern Uint8	FN1Prefix[ TP_MAXPREFIX ];
extern Uint8	FN1Drive[ MAXDRIVE ];
extern Uint8	FN1Dir[ MAXDIR ];
extern Uint8	FN1FileName[ MAXFILE + MAXEXT ];	// File Name, '.', Plus Extension

extern Uint8	FN1Template[ 12 ];		// Template Buffer


/* Components And Template For Path Number 2
-------------------------------------------- */
extern Uint8	FN2Prefix[ TP_MAXPREFIX ];
extern Uint8	FN2Drive[ MAXDRIVE ];
extern Uint8	FN2Dir[ MAXDIR ];
extern Uint8	FN2FileName[ MAXFILE + MAXEXT ];	// File Name, '.', Plus Extension

extern Uint8	FN2Template[ 12 ];		// Template Buffer


/* Function Prototypes
---------------------- */
int	fsGetDOSVars( void );
int	fsInitCDS( Uint8 driveNum );

void	fsFailCall( Uint16 ErrCode );

int	fsOurDrive( Uint8 far *TruePath );
int	fsCallForUs( char DriveNo );

void	fsMakeTemplate( Uint8 *Template, Uint8 *FileName );
int	fsMatchTemplate( Uint8 *Template, Uint8 *FileName );

void	fsSplitPath( Uint8 far *path,
			Uint8 *prefix, Uint8 *drive, Uint8 *dir, Uint8 *fname );

Uint16	fsDate( void );
Uint16	fsTime( void );
void		fsSetSFTOwner( SFT31 far *theSFTPtr );
Uint16	fsDecSFTUseCount( SFT31 far *theSFTPtr );

#ifdef __cplusplus
extern "C" {
#endif

void far ISRInit( void );

#ifdef __cplusplus
    #define __CPPARGS ...
#else
    #define __CPPARGS
#endif

void interrupt far Int2FSvc(__CPPARGS);
void interrupt far Int6XSvc(__CPPARGS);

#ifdef __cplusplus
}
#endif

#endif

